/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.modules.forms.ui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.olat.core.CoreSpringFactory;
import org.olat.core.commons.persistence.DefaultResultInfos;
import org.olat.core.commons.persistence.ResultInfos;
import org.olat.core.commons.persistence.SortKey;
import org.olat.core.gui.components.form.flexible.elements.FlexiTableFilter;
import org.olat.core.gui.components.form.flexible.impl.elements.table.FlexiTableDataSourceDelegate;
import org.olat.modules.forms.EvaluationFormManager;
import org.olat.modules.forms.EvaluationFormSession;
import org.olat.modules.forms.EvaluationFormSessionRef;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 
 * Initial date: 25.05.2018<br>
 * @author uhensler, urs.hensler@frentix.com, http://www.frentix.com
 *
 */
public class SessionSelectionDataSource implements FlexiTableDataSourceDelegate<SessionSelectionRow> {
	
	private final List<? extends EvaluationFormSessionRef> sessionRefs;
	
	@Autowired
	private EvaluationFormManager evaluationFormManager;
	
	public SessionSelectionDataSource(List<? extends EvaluationFormSessionRef> sessionRefs) {
		this.sessionRefs = sessionRefs;
		CoreSpringFactory.autowireObject(this);
	}

	@Override
	public int getRowCount() {
		return sessionRefs.size();
	}

	@Override
	public List<SessionSelectionRow> reload(List<SessionSelectionRow> rows) {
		return Collections.emptyList();
	}

	@Override
	public ResultInfos<SessionSelectionRow> getRows(String query, List<FlexiTableFilter> filters,
			List<String> condQueries, int firstResult, int maxResults, SortKey... orderBy) {
		
		List<EvaluationFormSession> sessions = evaluationFormManager.loadSessionsByKey(sessionRefs, firstResult, maxResults, orderBy);
		List<SessionSelectionRow> rows = new ArrayList<>();
		for (EvaluationFormSession session: sessions) {
			rows.add(new SessionSelectionRow(session));
		}

		return new DefaultResultInfos<>(firstResult + rows.size(), -1, rows);
	}

}
